@@ -160,6 +160,24 @@ struct apr_allocator_t {
  * Allocator
  */
 
+static APR_INLINE
+void allocator_lock(apr_allocator_t *allocator)
+{
+#if APR_HAS_THREADS
+    if (allocator->mutex)
+        apr_thread_mutex_lock(allocator->mutex);
+#endif /* APR_HAS_THREADS */
+}
+
+static APR_INLINE
+void allocator_unlock(apr_allocator_t *allocator)
+{
+#if APR_HAS_THREADS
+    if (allocator->mutex)
+        apr_thread_mutex_unlock(allocator->mutex);
+#endif /* APR_HAS_THREADS */
+}
+
 APR_DECLARE(apr_status_t) apr_allocator_create(apr_allocator_t **allocator)
 {
     apr_allocator_t *new_allocator;
@@ -179,7 +197,7 @@ APR_DECLARE(apr_status_t) apr_allocator_create(apr_allocator_t **allocator)
 
 APR_DECLARE(void) apr_allocator_destroy(apr_allocator_t *allocator)
 {
-    apr_uint32_t index;
+    apr_size_t index;
     apr_memnode_t *node, **ref;
 
     for (index = 0; index < MAX_INDEX; index++) {
@@ -226,16 +244,10 @@ APR_DECLARE(apr_pool_t *) apr_allocator_owner_get(apr_allocator_t *allocator)
 APR_DECLARE(void) apr_allocator_max_free_set(apr_allocator_t *allocator,
                                              apr_size_t in_size)
 {
-    apr_uint32_t max_free_index;
+    apr_size_t max_free_index;
     apr_size_t size = in_size;
 
-#if APR_HAS_THREADS
-    apr_thread_mutex_t *mutex;
-
-    mutex = apr_allocator_mutex_get(allocator);
-    if (mutex != NULL)
-        apr_thread_mutex_lock(mutex);
-#endif /* APR_HAS_THREADS */
+    allocator_lock(allocator);
 
     max_free_index = APR_ALIGN(size, BOUNDARY_SIZE) >> BOUNDARY_INDEX;
     allocator->current_free_index += max_free_index;
@@ -244,10 +256,7 @@ APR_DECLARE(void) apr_allocator_max_free_set(apr_allocator_t *allocator,
     if (allocator->current_free_index > max_free_index)
         allocator->current_free_index = max_free_index;
 
-#if APR_HAS_THREADS
-    if (mutex != NULL)
-        apr_thread_mutex_unlock(mutex);
-#endif
+    allocator_unlock(allocator);
 }
 
 static APR_INLINE
@@ -280,7 +289,7 @@ static APR_INLINE
 apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
 {
     apr_memnode_t *node, **ref;
-    apr_uint32_t max_index;
+    apr_size_t max_index;
     apr_size_t size, i, index;
 
     /* Round up the block size to the next boundary, but always
@@ -304,10 +313,7 @@ apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
      * our node will fit into.
      */
     if (index <= allocator->max_index) {
-#if APR_HAS_THREADS
-        if (allocator->mutex)
-            apr_thread_mutex_lock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_lock(allocator);
 
         /* Walk the free list to see if there are
          * any nodes on it of the requested size
@@ -338,7 +344,7 @@ apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
                     ref--;
                     max_index--;
                 }
-                while (*ref == NULL && max_index > 0);
+                while (*ref == NULL && max_index);
 
                 allocator->max_index = max_index;
             }
@@ -347,28 +353,19 @@ apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
             if (allocator->current_free_index > allocator->max_free_index)
                 allocator->current_free_index = allocator->max_free_index;
 
-#if APR_HAS_THREADS
-            if (allocator->mutex)
-                apr_thread_mutex_unlock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+            allocator_unlock(allocator);
 
             goto have_node;
         }
 
-#if APR_HAS_THREADS
-        if (allocator->mutex)
-            apr_thread_mutex_unlock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_unlock(allocator);
     }
 
     /* If we found nothing, seek the sink (at index 0), if
      * it is not empty.
      */
     else if (allocator->free[0]) {
-#if APR_HAS_THREADS
-        if (allocator->mutex)
-            apr_thread_mutex_lock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_lock(allocator);
 
         /* Walk the free list to see if there are
          * any nodes on it of the requested size
@@ -384,18 +381,12 @@ apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
             if (allocator->current_free_index > allocator->max_free_index)
                 allocator->current_free_index = allocator->max_free_index;
 
-#if APR_HAS_THREADS
-            if (allocator->mutex)
-                apr_thread_mutex_unlock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+            allocator_unlock(allocator);
 
             goto have_node;
         }
 
-#if APR_HAS_THREADS
-        if (allocator->mutex)
-            apr_thread_mutex_unlock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_unlock(allocator);
     }
 
     /* If we haven't got a suitable node, malloc a new one
@@ -419,7 +410,7 @@ apr_memnode_t *allocator_alloc(apr_allocator_t *allocator, apr_size_t in_size)
         return NULL;
     }
 #endif
-    node->index = index;
+    node->index = (apr_uint32_t)index;
     node->endp = (char *)node + size;
 
 have_node:
@@ -435,13 +426,10 @@ static APR_INLINE
 void allocator_free(apr_allocator_t *allocator, apr_memnode_t *node)
 {
     apr_memnode_t *next, *freelist = NULL;
-    apr_uint32_t index, max_index;
-    apr_uint32_t max_free_index, current_free_index;
+    apr_size_t index, max_index;
+    apr_size_t max_free_index, current_free_index;
 
-#if APR_HAS_THREADS
-    if (allocator->mutex)
-        apr_thread_mutex_lock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+    allocator_lock(allocator);
 
     max_index = allocator->max_index;
     max_free_index = allocator->max_free_index;
@@ -492,10 +480,7 @@ void allocator_free(apr_allocator_t *allocator, apr_memnode_t *node)
     allocator->max_index = max_index;
     allocator->current_free_index = current_free_index;
 
-#if APR_HAS_THREADS
-    if (allocator->mutex)
-        apr_thread_mutex_unlock(allocator->mutex);
-#endif /* APR_HAS_THREADS */
+    allocator_unlock(allocator);
 
     while (freelist != NULL) {
         node = freelist;
@@ -559,7 +544,7 @@ typedef struct debug_node_t debug_node_t;
 
 struct debug_node_t {
     debug_node_t *next;
-    apr_uint32_t  index;
+    apr_size_t    index;
     void         *beginp[64];
     void         *endp[64];
 };
@@ -636,6 +621,12 @@ static apr_allocator_t *global_allocator = NULL;
 
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE_ALL)
 static apr_file_t *file_stderr = NULL;
+static apr_status_t apr_pool_cleanup_file_stderr(void *data)
+{
+    file_stderr = NULL;
+    return APR_SUCCESS;
+}
+
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE_ALL) */
 
 /*
@@ -870,7 +861,7 @@ APR_DECLARE(void *) apr_palloc(apr_pool_t *pool, apr_size_t in_size)
     free_index = (APR_ALIGN(active->endp - active->first_avail + 1,
                             BOUNDARY_SIZE) - BOUNDARY_SIZE) >> BOUNDARY_INDEX;
 
-    active->free_index = free_index;
+    active->free_index = (apr_uint32_t)free_index;
     node = active->next;
     if (free_index >= node->free_index)
         goto have_mem;
@@ -1005,20 +996,12 @@ APR_DECLARE(void) apr_pool_destroy(apr_pool_t *pool)
 
     /* Remove the pool from the parents child list */
     if (pool->parent) {
-#if APR_HAS_THREADS
-        apr_thread_mutex_t *mutex;
-
-        if ((mutex = apr_allocator_mutex_get(pool->parent->allocator)) != NULL)
-            apr_thread_mutex_lock(mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_lock(pool->parent->allocator);
 
         if ((*pool->ref = pool->sibling) != NULL)
             pool->sibling->ref = pool->ref;
 
-#if APR_HAS_THREADS
-        if (mutex)
-            apr_thread_mutex_unlock(mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_unlock(pool->parent->allocator);
     }
 
     /* Find the block attached to the pool structure.  Save a copy of the
@@ -1121,12 +1104,7 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex(apr_pool_t **newpool,
 #endif /* defined(NETWARE) */
 
     if ((pool->parent = parent) != NULL) {
-#if APR_HAS_THREADS
-        apr_thread_mutex_t *mutex;
-
-        if ((mutex = apr_allocator_mutex_get(parent->allocator)) != NULL)
-            apr_thread_mutex_lock(mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_lock(parent->allocator);
 
         if ((pool->sibling = parent->child) != NULL)
             pool->sibling->ref = &pool->sibling;
@@ -1134,10 +1112,7 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex(apr_pool_t **newpool,
         parent->child = pool;
         pool->ref = &parent->child;
 
-#if APR_HAS_THREADS
-        if (mutex)
-            apr_thread_mutex_unlock(mutex);
-#endif /* APR_HAS_THREADS */
+        allocator_unlock(parent->allocator);
     }
     else {
         pool->sibling = NULL;
@@ -1286,7 +1261,7 @@ static int psprintf_flush(apr_vformatter_buff_t *vbuff)
         free_index = (APR_ALIGN(active->endp - active->first_avail + 1,
                                 BOUNDARY_SIZE) - BOUNDARY_SIZE) >> BOUNDARY_INDEX;
 
-        active->free_index = free_index;
+        active->free_index = (apr_uint32_t)free_index;
         node = active->next;
         if (free_index < node->free_index) {
             do {
@@ -1363,7 +1338,7 @@ APR_DECLARE(char *) apr_pvsprintf(apr_pool_t *pool, const char *fmt, va_list ap)
     apr_size_t free_index;
 
     pool_concurrency_set_used(pool);
-    ps.node = active = pool->active;
+    ps.node = pool->active;
     ps.pool = pool;
     ps.vbuff.curpos  = ps.node->first_avail;
 
@@ -1442,7 +1417,7 @@ APR_DECLARE(char *) apr_pvsprintf(apr_pool_t *pool, const char *fmt, va_list ap)
     free_index = (APR_ALIGN(active->endp - active->first_avail + 1,
                             BOUNDARY_SIZE) - BOUNDARY_SIZE) >> BOUNDARY_INDEX;
 
-    active->free_index = free_index;
+    active->free_index = (apr_uint32_t)free_index;
     node = active->next;
 
     if (free_index >= node->free_index) {
@@ -1480,6 +1455,41 @@ error:
  * Debug helper functions
  */
 
+static APR_INLINE
+void pool_lock(apr_pool_t *pool)
+{
+#if APR_HAS_THREADS
+    apr_thread_mutex_lock(pool->mutex);
+#endif /* APR_HAS_THREADS */
+}
+
+static APR_INLINE
+void pool_unlock(apr_pool_t *pool)
+{
+#if APR_HAS_THREADS
+    apr_thread_mutex_unlock(pool->mutex);
+#endif /* APR_HAS_THREADS */
+}
+
+#if APR_HAS_THREADS
+static APR_INLINE
+apr_thread_mutex_t *parent_lock(apr_pool_t *pool)
+{
+    if (pool->parent) {
+        apr_thread_mutex_lock(pool->parent->mutex);
+        return pool->parent->mutex;
+    }
+    return NULL;
+}
+
+static APR_INLINE
+void parent_unlock(apr_thread_mutex_t *mutex)
+{
+    if (mutex) {
+        apr_thread_mutex_unlock(mutex);
+    }
+}
+#endif /* APR_HAS_THREADS */
 
 /*
  * Walk the pool tree rooted at pool, depth first.  When fn returns
@@ -1497,11 +1507,7 @@ static int apr_pool_walk_tree(apr_pool_t *pool,
     if (rv)
         return rv;
 
-#if APR_HAS_THREADS
-    if (pool->mutex) {
-        apr_thread_mutex_lock(pool->mutex);
-                        }
-#endif /* APR_HAS_THREADS */
+    pool_lock(pool);
 
     child = pool->child;
     while (child) {
@@ -1512,11 +1518,7 @@ static int apr_pool_walk_tree(apr_pool_t *pool,
         child = child->sibling;
     }
 
-#if APR_HAS_THREADS
-    if (pool->mutex) {
-        apr_thread_mutex_unlock(pool->mutex);
-    }
-#endif /* APR_HAS_THREADS */
+    pool_unlock(pool);
 
     return rv;
 }
@@ -1538,6 +1540,7 @@ static void apr_pool_log_event(apr_pool_t *pool, const char *event,
                 "(%10lu/%10lu/%10lu) "
                 "0x%pp \"%s\" "
                 "<%s> "
+                "0x%pp "
                 "(%u/%u/%u) "
                 "\n",
                 (unsigned long)getpid(),
@@ -1550,6 +1553,7 @@ static void apr_pool_log_event(apr_pool_t *pool, const char *event,
                 (unsigned long)apr_pool_num_bytes(global_pool, 1),
                 pool, pool->tag,
                 file_line,
+                pool->parent,
                 pool->stat_alloc, pool->stat_total_alloc, pool->stat_clear);
         }
         else {
@@ -1594,7 +1598,7 @@ static int apr_pool_is_child_of(apr_pool_t *pool, apr_pool_t *parent)
 }
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_LIFETIME) */
 
-static void apr_pool_check_integrity(apr_pool_t *pool)
+static void apr_pool_check_lifetime(apr_pool_t *pool)
 {
     /* Rule of thumb: use of the global pool is always
      * ok, since the only user is apr_pools.c.  Unless
@@ -1618,7 +1622,10 @@ static void apr_pool_check_integrity(apr_pool_t *pool)
         abort();
     }
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_LIFETIME) */
+}
 
+static void apr_pool_check_owner(apr_pool_t *pool)
+{
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_OWNER)
 #if APR_HAS_THREADS
     if (!apr_os_thread_equal(pool->owner, apr_os_thread_current())) {
@@ -1632,6 +1639,11 @@ static void apr_pool_check_integrity(apr_pool_t *pool)
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_OWNER) */
 }
 
+static void apr_pool_check_integrity(apr_pool_t *pool)
+{
+    apr_pool_check_lifetime(pool);
+    apr_pool_check_owner(pool);
+}
 
 /*
  * Initialization (debug)
@@ -1702,9 +1714,16 @@ APR_DECLARE(apr_status_t) apr_pool_initialize(void)
             "/TID"
 #endif /* APR_HAS_THREADS */
             "] ACTION  (SIZE      /POOL SIZE /TOTAL SIZE) "
-            "POOL       \"TAG\" <__FILE__:__LINE__> (ALLOCS/TOTAL ALLOCS/CLEARS)\n");
+            "POOL       \"TAG\" <__FILE__:__LINE__> PARENT     (ALLOCS/TOTAL ALLOCS/CLEARS)\n");
 
         apr_pool_log_event(global_pool, "GLOBAL", __FILE__ ":apr_pool_initialize", 0);
+
+        /* Add a cleanup handler that sets the debug log file handle
+         * to NULL, otherwise we'll try to log the global pool
+         * destruction event with predictably disastrous results. */
+        apr_pool_cleanup_register(global_pool, NULL,
+                                  apr_pool_cleanup_file_stderr,
+                                  apr_pool_cleanup_null);
     }
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE_ALL) */
 
@@ -1814,12 +1833,18 @@ APR_DECLARE(void *) apr_pcalloc_debug(apr_pool_t *pool, apr_size_t size,
 static void pool_clear_debug(apr_pool_t *pool, const char *file_line)
 {
     debug_node_t *node;
-    apr_uint32_t index;
+    apr_size_t index;
 
     /* Run pre destroy cleanups */
     run_cleanups(&pool->pre_cleanups);
     pool->pre_cleanups = NULL;
 
+    /*
+     * Now that we have given the pre cleanups the chance to kill of any
+     * threads using the pool, the owner must be correct.
+     */
+    apr_pool_check_owner(pool);
+
     /* Destroy the subpools.  The subpools will detach themselves from
      * this pool thus this loop is safe and easy.
      */
@@ -1859,79 +1884,65 @@ static void pool_clear_debug(apr_pool_t *pool, const char *file_line)
 
     pool->stat_alloc = 0;
     pool->stat_clear++;
+
+#if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
+    apr_pool_log_event(pool, "CLEARED", file_line, 1);
+#endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
 }
 
 APR_DECLARE(void) apr_pool_clear_debug(apr_pool_t *pool,
                                        const char *file_line)
 {
 #if APR_HAS_THREADS
-    apr_thread_mutex_t *mutex = NULL;
+    apr_thread_mutex_t *mutex;
 #endif
 
-    apr_pool_check_integrity(pool);
+    apr_pool_check_lifetime(pool);
+
+#if APR_HAS_THREADS
+    /* Lock the parent mutex before clearing so that if we have our
+     * own mutex it won't be accessed by apr_pool_walk_tree after
+     * it has been destroyed.
+     */
+    mutex = parent_lock(pool);
+#endif
 
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
     apr_pool_log_event(pool, "CLEAR", file_line, 1);
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
 
-#if APR_HAS_THREADS
-    if (pool->parent != NULL)
-        mutex = pool->parent->mutex;
-
-    /* Lock the parent mutex before clearing so that if we have our
-     * own mutex it won't be accessed by apr_pool_walk_tree after
-     * it has been destroyed.
-     */
-    if (mutex != NULL && mutex != pool->mutex) {
-        apr_thread_mutex_lock(mutex);
-    }
-#endif
-
     pool_clear_debug(pool, file_line);
 
 #if APR_HAS_THREADS
     /* If we had our own mutex, it will have been destroyed by
-     * the registered cleanups.  Recreate the mutex.  Unlock
-     * the mutex we obtained above.
+     * the registered cleanups.  Recreate it.
      */
     if (mutex != pool->mutex) {
+        /*
+         * Prevent apr_palloc() in apr_thread_mutex_create() from trying to
+         * use the destroyed mutex.
+         */
+        pool->mutex = NULL;
         (void)apr_thread_mutex_create(&pool->mutex,
                                       APR_THREAD_MUTEX_NESTED, pool);
-
-        if (mutex != NULL)
-            (void)apr_thread_mutex_unlock(mutex);
     }
+
+    /* Unlock the mutex we obtained above */
+    parent_unlock(mutex);
 #endif /* APR_HAS_THREADS */
 }
 
 static void pool_destroy_debug(apr_pool_t *pool, const char *file_line)
 {
-    apr_pool_check_integrity(pool);
-
-#if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
-    apr_pool_log_event(pool, "DESTROY", file_line, 1);
-#endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
-
     pool_clear_debug(pool, file_line);
 
-    /* Remove the pool from the parents child list */
-    if (pool->parent) {
-#if APR_HAS_THREADS
-        apr_thread_mutex_t *mutex;
-
-        if ((mutex = pool->parent->mutex) != NULL)
-            apr_thread_mutex_lock(mutex);
-#endif /* APR_HAS_THREADS */
-
-        if ((*pool->ref = pool->sibling) != NULL)
-            pool->sibling->ref = pool->ref;
-
-#if APR_HAS_THREADS
-        if (mutex)
-            apr_thread_mutex_unlock(mutex);
-#endif /* APR_HAS_THREADS */
+    /* Remove the pool from the parent's child list */
+    if (pool->parent != NULL
+        && (*pool->ref = pool->sibling) != NULL) {
+        pool->sibling->ref = pool->ref;
     }
 
+    /* Destroy the allocator if the pool owns it */
     if (pool->allocator != NULL
         && apr_allocator_owner_get(pool->allocator) == pool) {
         apr_allocator_destroy(pool->allocator);
@@ -1944,6 +1955,12 @@ static void pool_destroy_debug(apr_pool_t *pool, const char *file_line)
 APR_DECLARE(void) apr_pool_destroy_debug(apr_pool_t *pool,
                                          const char *file_line)
 {
+#if APR_HAS_THREADS
+    apr_thread_mutex_t *mutex;
+#endif
+
+    apr_pool_check_lifetime(pool);
+
     if (pool->joined) {
         /* Joined pools must not be explicitly destroyed; the caller
          * has broken the guarantee. */
@@ -1954,7 +1971,24 @@ APR_DECLARE(void) apr_pool_destroy_debug(apr_pool_t *pool,
 
         abort();
     }
+
+#if APR_HAS_THREADS
+    /* Lock the parent mutex before destroying so that it's not accessed
+     * concurrently by apr_pool_walk_tree.
+     */
+    mutex = parent_lock(pool);
+#endif
+
+#if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
+    apr_pool_log_event(pool, "DESTROY", file_line, 1);
+#endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
+
     pool_destroy_debug(pool, file_line);
+
+#if APR_HAS_THREADS
+    /* Unlock the mutex we obtained above */
+    parent_unlock(mutex);
+#endif /* APR_HAS_THREADS */
 }
 
 APR_DECLARE(apr_status_t) apr_pool_create_ex_debug(apr_pool_t **newpool,
@@ -1971,7 +2005,7 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex_debug(apr_pool_t **newpool,
         parent = global_pool;
     }
     else {
-       apr_pool_check_integrity(parent);
+       apr_pool_check_lifetime(parent);
 
        if (!allocator)
            allocator = parent->allocator;
@@ -1994,37 +2028,8 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex_debug(apr_pool_t **newpool,
     pool->tag = file_line;
     pool->file_line = file_line;
 
-    if ((pool->parent = parent) != NULL) {
 #if APR_HAS_THREADS
-        if (parent->mutex)
-            apr_thread_mutex_lock(parent->mutex);
-#endif /* APR_HAS_THREADS */
-        if ((pool->sibling = parent->child) != NULL)
-            pool->sibling->ref = &pool->sibling;
-
-        parent->child = pool;
-        pool->ref = &parent->child;
-
-#if APR_HAS_THREADS
-        if (parent->mutex)
-            apr_thread_mutex_unlock(parent->mutex);
-#endif /* APR_HAS_THREADS */
-    }
-    else {
-        pool->sibling = NULL;
-        pool->ref = NULL;
-    }
-
-#if APR_HAS_THREADS
-    pool->owner = apr_os_thread_current();
-#endif /* APR_HAS_THREADS */
-#ifdef NETWARE
-    pool->owner_proc = (apr_os_proc_t)getnlmhandle();
-#endif /* defined(NETWARE) */
-
-
     if (parent == NULL || parent->allocator != allocator) {
-#if APR_HAS_THREADS
         apr_status_t rv;
 
         /* No matter what the creation flags say, always create
@@ -2040,21 +2045,41 @@ APR_DECLARE(apr_status_t) apr_pool_create_ex_debug(apr_pool_t **newpool,
             free(pool);
             return rv;
         }
-#endif /* APR_HAS_THREADS */
     }
     else {
-#if APR_HAS_THREADS
-        if (parent)
-            pool->mutex = parent->mutex;
+        pool->mutex = parent->mutex;
+    }
 #endif /* APR_HAS_THREADS */
+
+    if ((pool->parent = parent) != NULL) {
+        pool_lock(parent);
+
+        if ((pool->sibling = parent->child) != NULL)
+            pool->sibling->ref = &pool->sibling;
+
+        parent->child = pool;
+        pool->ref = &parent->child;
+
+        pool_unlock(parent);
+    }
+    else {
+        pool->sibling = NULL;
+        pool->ref = NULL;
     }
 
-    *newpool = pool;
+#if APR_HAS_THREADS
+    pool->owner = apr_os_thread_current();
+#endif /* APR_HAS_THREADS */
+#ifdef NETWARE
+    pool->owner_proc = (apr_os_proc_t)getnlmhandle();
+#endif /* defined(NETWARE) */
 
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
     apr_pool_log_event(pool, "CREATE", file_line, 1);
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
 
+    *newpool = pool;
+
     return APR_SUCCESS;
 }
 
@@ -2090,6 +2115,26 @@ APR_DECLARE(apr_status_t) apr_pool_create_unmanaged_ex_debug(apr_pool_t **newpoo
     pool->tag = file_line;
     pool->file_line = file_line;
 
+#if APR_HAS_THREADS
+    {
+        apr_status_t rv;
+
+        /* No matter what the creation flags say, always create
+         * a lock.  Without it integrity_check and apr_pool_num_bytes
+         * blow up (because they traverse pools child lists that
+         * possibly belong to another thread, in combination with
+         * the pool having no lock).  However, this might actually
+         * hide problems like creating a child pool of a pool
+         * belonging to another thread.
+         */
+        if ((rv = apr_thread_mutex_create(&pool->mutex,
+                APR_THREAD_MUTEX_NESTED, pool)) != APR_SUCCESS) {
+            free(pool);
+            return rv;
+        }
+    }
+#endif /* APR_HAS_THREADS */
+
 #if APR_HAS_THREADS
     pool->owner = apr_os_thread_current();
 #endif /* APR_HAS_THREADS */
@@ -2108,32 +2153,12 @@ APR_DECLARE(apr_status_t) apr_pool_create_unmanaged_ex_debug(apr_pool_t **newpoo
     }
     pool->allocator = pool_allocator;
 
-    if (pool->allocator != allocator) {
-#if APR_HAS_THREADS
-        apr_status_t rv;
-
-        /* No matter what the creation flags say, always create
-         * a lock.  Without it integrity_check and apr_pool_num_bytes
-         * blow up (because they traverse pools child lists that
-         * possibly belong to another thread, in combination with
-         * the pool having no lock).  However, this might actually
-         * hide problems like creating a child pool of a pool
-         * belonging to another thread.
-         */
-        if ((rv = apr_thread_mutex_create(&pool->mutex,
-                APR_THREAD_MUTEX_NESTED, pool)) != APR_SUCCESS) {
-            free(pool);
-            return rv;
-        }
-#endif /* APR_HAS_THREADS */
-    }
-
-    *newpool = pool;
-
 #if (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE)
     apr_pool_log_event(pool, "CREATEU", file_line, 1);
 #endif /* (APR_POOL_DEBUG & APR_POOL_DEBUG_VERBOSE) */
 
+    *newpool = pool;
+
     return APR_SUCCESS;
 }
 
@@ -2230,7 +2255,7 @@ static int pool_find(apr_pool_t *pool, void *data)
 {
     void **pmem = (void **)data;
     debug_node_t *node;
-    apr_uint32_t index;
+    apr_size_t index;
 
     node = pool->nodes;
 
@@ -2263,7 +2288,7 @@ static int pool_num_bytes(apr_pool_t *pool, void *data)
 {
     apr_size_t *psize = (apr_size_t *)data;
     debug_node_t *node;
-    apr_uint32_t index;
+    apr_size_t index;
 
     node = pool->nodes;
 
@@ -2480,7 +2505,7 @@ APR_DECLARE(void) apr_pool_cleanup_register(apr_pool_t *p, const void *data,
                       apr_status_t (*plain_cleanup_fn)(void *data),
                       apr_status_t (*child_cleanup_fn)(void *data))
 {
-    cleanup_t *c;
+    cleanup_t *c = NULL;
 
 #if APR_POOL_DEBUG
     apr_pool_check_integrity(p);
@@ -2500,12 +2525,18 @@ APR_DECLARE(void) apr_pool_cleanup_register(apr_pool_t *p, const void *data,
         c->next = p->cleanups;
         p->cleanups = c;
     }
+
+#if APR_POOL_DEBUG
+    if (!c || !c->plain_cleanup_fn || !c->child_cleanup_fn) {
+        abort();
+    }
+#endif /* APR_POOL_DEBUG */
 }
 
 APR_DECLARE(void) apr_pool_pre_cleanup_register(apr_pool_t *p, const void *data,
                       apr_status_t (*plain_cleanup_fn)(void *data))
 {
-    cleanup_t *c;
+    cleanup_t *c = NULL;
 
 #if APR_POOL_DEBUG
     apr_pool_check_integrity(p);
@@ -2524,6 +2555,12 @@ APR_DECLARE(void) apr_pool_pre_cleanup_register(apr_pool_t *p, const void *data,
         c->next = p->pre_cleanups;
         p->pre_cleanups = c;
     }
+
+#if APR_POOL_DEBUG
+    if (!c || !c->plain_cleanup_fn) {
+        abort();
+    }
+#endif /* APR_POOL_DEBUG */
 }
 
 APR_DECLARE(void) apr_pool_cleanup_kill(apr_pool_t *p, const void *data,
