@@ -606,7 +606,7 @@ static void file_contents_equal(abts_case *tc,
 static void test_puts(abts_case *tc, void *data)
 {
     apr_file_t *f;
-    const char *fname = "data/testputs.txt";
+    const char *fname = "data/testputs.dat";
 
     APR_ASSERT_SUCCESS(tc, "open file for writing",
                        apr_file_open(&f, fname, 
@@ -629,7 +629,7 @@ static void test_writev(abts_case *tc, void *data)
     apr_file_t *f;
     apr_size_t nbytes;
     struct iovec vec[5];
-    const char *fname = "data/testwritev.txt";
+    const char *fname = "data/testwritev.dat";
 
     APR_ASSERT_SUCCESS(tc, "open file for writing",
                        apr_file_open(&f, fname, 
@@ -671,7 +671,7 @@ static void test_writev_full(abts_case *tc, void *data)
     apr_file_t *f;
     apr_size_t nbytes;
     struct iovec vec[5];
-    const char *fname = "data/testwritev_full.txt";
+    const char *fname = "data/testwritev_full.dat";
 
     APR_ASSERT_SUCCESS(tc, "open file for writing",
                        apr_file_open(&f, fname, 
@@ -1219,6 +1219,47 @@ static void test_xthread(abts_case *tc, void *data)
     apr_file_close(f);
 }
 
+static void test_datasync_on_file(abts_case *tc, void *data)
+{
+    apr_status_t rv;
+    apr_file_t *f;
+    const char *fname = DIRNAME "/testtest_datasync.dat";
+    apr_size_t bytes_written;
+
+    apr_file_remove(fname, p);
+
+    rv = apr_file_open(&f, fname, APR_FOPEN_CREATE | APR_FOPEN_WRITE,
+                       APR_FPROT_OS_DEFAULT, p);
+    APR_ASSERT_SUCCESS(tc, "open test file for writing", rv);
+    rv = apr_file_write_full(f, "abcdef", 6, &bytes_written);
+    APR_ASSERT_SUCCESS(tc, "write to file", rv);
+    rv = apr_file_datasync(f);
+    APR_ASSERT_SUCCESS(tc, "sync file contents", rv);
+    apr_file_close(f);
+
+    apr_file_remove(fname, p);
+}
+
+static void test_datasync_on_stream(abts_case *tc, void *data)
+{
+    apr_status_t rv;
+    apr_file_t *f;
+    apr_size_t bytes_written;
+
+    rv = apr_file_open_stdout(&f, p);
+    APR_ASSERT_SUCCESS(tc, "open stdout", rv);
+    rv = apr_file_write_full(f, "abcdef\b\b\b\b\b\b\b", 12, &bytes_written);
+    APR_ASSERT_SUCCESS(tc, "write to stdout", rv);
+    rv = apr_file_datasync(f);
+    if (rv != APR_SUCCESS) {
+#if defined(__APPLE__)
+        ABTS_INT_EQUAL(tc, ENOTSUP, rv);
+#else
+        ABTS_INT_EQUAL(tc, APR_EINVAL, rv);
+#endif
+    }
+}
+
 abts_suite *testfile(abts_suite *suite)
 {
     suite = ADD_SUITE(suite)
@@ -1263,6 +1304,8 @@ abts_suite *testfile(abts_suite *suite)
     abts_run_test(suite, test_fail_read_flush, NULL);
     abts_run_test(suite, test_buffer_set_get, NULL);
     abts_run_test(suite, test_xthread, NULL);
+    abts_run_test(suite, test_datasync_on_file, NULL);
+    abts_run_test(suite, test_datasync_on_stream, NULL);
 
     return suite;
 }
